#!/bin/sh

EXT2_O=/usr/src/linux/fs/ext2/ext2.o
EXT3_O=/usr/src/linux/fs/ext3/ext3.o
SNAPFS_O=../snapfs22/snapfs.o
CACHE_TYPE="ext2"
CURRENT=/mnt/current
CLONE=/mnt/clone
SNAPFILE=/tmp/fssnap
SNAPCTL=../util/snapctl
SLEEP=1
CURR_DIR=`pwd`
CURRENT_LOOPDEV=/dev/loop0

if [ X$2 != X"" ]; then
	CACHE_TYPE=$2
fi

mount_current() {
        mount -t snap_current -o loop,table=0,cache_type=$CACHE_TYPE $SNAPFILE $CURRENT
        echo "mount -t snap_current -o loop,table=0,cache_type=$CACHE_TYPE $SNAPFILE $CURRENT"
}
mount_clone() {
    mount -t snap_clone -o dev=$CURRENT_LOOPDEV,name=clone1 none $CLONE
    echo "mount -t snap_clone -o dev=$CURRENT_LOOPDEV,name=clone1 none $CLONE"
}

disk_usage() {
    echo ""
    echo "Show disk usage of current/clone file system ( run 'df' )"
    echo ""
    df -t snap_current -t snap_clone
    echo ""
}

check_disk(){

    if [ X$CACHE_TYPE = X"ext2" ]; then
	    echo ""
	    echo "Checking disk $SNAPFILE ( run e2fsck )"
	    echo ""
	    e2fsck -f $SNAPFILE -y
	    echo ""
    fi
}

show_stat() {
    disk_usage
    echo " the contents of $CURRENT(ls -li $CURRENT) is:"
    echo ""
    ls -li $CURRENT 
    echo ""
    echo " the contents of $CLONE(ls -li $CLONE) is:"
    ls -li $CLONE
    echo ""
}

readline() {
	echo "Press <enter> to continue"
	while [ 1 ]; do 
		read line;
		if [ X$line = X"" ]; then
			break
		fi
	done
}

mywait() {
	readline
	sleep $SLEEP
}

snap_common(){

    echo "Step 0: Preparation -- load necessary modules"
    if [ X"`lsmod |grep snapfs`" != X"" ]; then
	echo "snapfs module is not removed, please run '$0 stop' first"
	exit
    fi
    if [ ! -f $SNAPFS_O ]; then
	echo "snapfs module $SNAPFS_O doesn't exist, exit" 
	exit
    fi

    if [ ! -f $SNAPCTL ]; then
	echo "snapfs utility $SNAPCTL doesn't exist, exit" 
	exit
    fi

    if [ ! -d $CURRENT ]; then
	mkdir -p $CURRENT
    fi
    if [ ! -d $CLONE ]; then
	mkdir -p $CLONE
    fi

    mywait

    if [ -f $EXT2_O ] && [ X$CACHE_TYPE = X"ext2" ]; then
    	insmod $EXT2_O
    fi


    if [ -f $EXT3_O ] && [ X$CACHE_TYPE = X"ext3" ]; then
    	insmod $EXT3_O
    fi

    if [ -f $SNAPFS_O ]; then
    	insmod $SNAPFS_O
    else
    	modprobe snapfs
    fi

# create the original files
    echo "Step 1: Create sample files -- use file $SNAPFILE"
    mywait
    rm -f $SNAPFILE
    if [ -f $SNAPFILE ]; then 
	echo "$SNAPFILE exist, exit"
	exit 0
    fi

    if [ ! -f $SNAPFILE ]; then 
	dd if=/dev/zero of=$SNAPFILE bs=1024 count=10k >/dev/null 2>/dev/null
	/sbin/mke2fs -F $SNAPFILE >/dev/null 2>/dev/null
    fi

    mount -o loop $SNAPFILE $CURRENT

    if [ X$CACHE_TYPE = X"ext3" ]; then
	cd $CURRENT
	dd if=/dev/zero of=.journal bs=1024 count=1k >/dev/null 2>/dev/null
	chattr +i .journal
	cd /
	umount $CURRENT
	mount -t ext3 -o loop,journal=12 $SNAPFILE $CURRENT
    fi
    cd $CURRENT
    echo "yesterday" > hello
    echo "test" > bye
    echo "a" > a
    echo "b" > b
 #  touch a b
    ln -s hello link
    cd $CURR_DIR
    umount $CURRENT

    check_disk
	
# set up snaptables, install system, 
    echo "Step 2: Install snaptable -- one snapshot, index 1, name: clone1 "
    mywait

    rm -f /etc/snapcontrol/table0
    $SNAPCTL -f <<EOF >/dev/null
snaptable
table0
d
1
a
1
now
clone1
q
y
snapset 0 table0
quit
EOF

    echo "Step 3: Mount current and clone filesystem "
    mywait
    mount_current
    mount_clone

    # list original contents
    show_stat
   
    echo "Step 4: Make some changes on current filesystem"
    mywait

    echo "today" >> $CURRENT/hello
    cp /etc/hosts $CURRENT
    rm $CURRENT/a
    chmod 777 $CURRENT/b

    show_stat
    echo "Show the content of $CLONE/hello, you will see: yesterday"
    cat $CLONE/hello
    echo "Show the content of $CURRENT/hello, you will see: yesterday\ntoday"
    cat $CURRENT/hello
    echo "Show the content of $CURRENT/link, you will see: yesterday\ntoday"
    cat $CURRENT/link
    echo ""
}

snap_delete() {

    $SNAPCTL -f <<EOF > /dev/null
snapdelete clone1 table0
quit
EOF
    umount $CLONE
    umount $CURRENT
    mount_current

    echo "After Snap Delete :"
    show_stat
    echo "Show the content of $CURRENT/hello, you will see: yesterday\ntoday"
    cat $CURRENT/hello
    echo "Show the content of $CURRENT/link, you will see: yesterday\ntoday"
    cat $CURRENT/link
    echo ""
}

snap_rollback(){

    $SNAPCTL -f <<EOF > /dev/null
snaprollback clone1 table0 0
quit
EOF
    umount $CLONE
    umount $CURRENT
    mount_current

    echo "After Snap Rollback :"
    show_stat
    echo "Show the content of $CURRENT/hello, you will see: yesterday"
    cat $CURRENT/hello
    echo "Show the content of $CURRENT/link, you will see: yesterday"
    cat $CURRENT/link
    echo ""
}


clean_all() {
    umount $CLONE
    umount $CURRENT
    rmmod snapfs
    if [ X$CACHE_TYPE = X"ext2" ]; then
    	 rmmod ext2
    fi
    if [ X$CACHE_TYPE = X"ext3" ]; then
   	 rmmod ext3
    fi
 
    rmmod loop
}
 
case "$1" in

    start)
	echo " ==== SnapFS / Snap Delete Demo ==="
	snap_common
  	echo "Step 5: Snap Delete "
 	mywait
	snap_delete
	echo "Step 6: Clean up "
	mywait
	clean_all
	echo ""
	echo "After snap delete, you should see a clean file system"
	check_disk
	mywait
	echo " ==== SnapFS / Snap Rollback Demo ==="
	mywait
	snap_common
  	echo "Step 5: Snap Rollback "
	mywait
	snap_rollback
	echo "Step 6: Clean up "
	mywait
	clean_all
	echo ""
	echo "After snap rollback, you should see a clean file system"
	check_disk
    	echo "done"
    ;;

  delete)
	echo " ==== SnapFS / Snap Delete Demo ==="
	snap_common
  	echo "Step 5: Snap Delete "
 	mywait
	snap_delete
	echo ""
	echo "Step 6: Clean up "
	mywait
	clean_all
	echo ""
	echo "After snap delete, you should see a clean file system"
	check_disk
	echo "done"
    ;;
  rollback)
	echo " ==== SnapFS / Snap Rollback Demo ==="
	mywait
	snap_common
  	echo "Step 5: Snap Rollback "
	mywait
	snap_rollback
	echo "Step 6: Clean up "
	mywait
	clean_all
    	echo ""
	echo "After snap rollback, you should see a clean file system"
	check_disk
    	echo "done"
    ;;

    stop)
	clean_all
   ;;
    *)
    echo "Usage $0 { start, delete, rollback, stop} [ext3]"
    exit 1;
    
esac

exit 0
