/*
 *  fs/snap/snap.c
 *
 *  A snap shot file system.
 *
 */

#define EXPORT_SYMTAB


#define __NO_VERSION__
#include <linux/module.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include <linux/stat.h>
#include <linux/string.h>
#include <linux/locks.h>
#include <linux/quotaops.h>
#include <linux/list.h>
#include <linux/file.h>
#include <asm/bitops.h>
#include <asm/byteorder.h>

#include <linux/filter.h>
#include <linux/snapfs.h>
#include <linux/snapsupport.h>

static inline int inode_has_ea(struct inode *inode)
{               
        return (inode->u.ext2_i.i_file_acl != 0); 
}               

static int currentfs_readlink(struct dentry * dentry, char * buffer, int buflen)
{
	struct snap_cache *cache;
	int rc;
	struct inode_operations *iops;
	struct inode * inode = dentry->d_inode;
	int bpib = inode->i_sb->s_blocksize >> 9;
	__u32 save_i_blocks;

	ENTRY;

	cache = snap_find_cache(inode->i_dev);
	if ( !cache ) { 
		EXIT;
		return -EINVAL;
	}

	iops = filter_c2csiops(cache->cache_filter); 
	if (!iops ||
	    !iops->readlink) {
		rc = -EINVAL;
		goto exit;
	}

	save_i_blocks = inode->i_blocks;
	/* If this link has ea and its i_blocks is ea's block, 
	 * then we should treate it as a fast symlink 
	 */
	if( inode_has_ea(inode) && inode->i_blocks == bpib ) {
		inode->i_blocks = 0; 
	}
	rc = iops->readlink(dentry, buffer, buflen);
	
	if( inode->i_blocks != save_i_blocks ){
		inode->i_blocks = save_i_blocks;
		mark_inode_dirty(inode);
	}
	
exit:
	EXIT;
	return rc;
}

static struct dentry * currentfs_follow_link (	struct dentry * dentry, 
						struct dentry *base,
						unsigned int follow)
{
	struct snap_cache *cache;
	struct dentry * rc;
	struct inode_operations *iops;
	struct inode * inode = dentry->d_inode;
	int bpib = inode->i_sb->s_blocksize >> 9;
	__u32 save_i_blocks;

	ENTRY;

	cache = snap_find_cache(inode->i_dev);
	if ( !cache ) { 
		EXIT;
		return ERR_PTR(-EINVAL);
	}

	iops = filter_c2csiops(cache->cache_filter); 
	if (!iops ||
	    !iops->follow_link) {
		rc = ERR_PTR(-EINVAL);
		goto exit;
	}

	save_i_blocks = inode->i_blocks;
	/* If this link has ea and its i_blocks is ea's block, 
	 * then we should treate it as a fast symlink 
	 */
	if( inode_has_ea(inode) && inode->i_blocks == bpib ) {
		inode->i_blocks = 0; 
	}
	rc = iops->follow_link(dentry, base, follow);
	
	if( inode->i_blocks != save_i_blocks ){
		inode->i_blocks = save_i_blocks;
		mark_inode_dirty(inode);
	}
	
exit:
	EXIT;
	return rc;
}
struct inode_operations currentfs_sym_iops = {
	readlink:	currentfs_readlink,
	follow_link:	currentfs_follow_link
};

struct file_operations currentfs_sym_fops;
