/* fork_test.c
 *
 * Andreas Dilger <adilger@turbolinux.com>
 * Harrison Xing <harrisonx@mountainviewdata.com>
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
//#include <linux/ext2_fs.h>
#include </usr/src/linux/include/linux/ext2_fs.h>
#include <fcntl.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
	int fd, err;
	int ioc;
	long val;
	ino_t ino;

	int index=1;

	if (argc < 3) {
		fprintf(stderr, "usage: %s <file> <cmdflag>\n", 
			argv[0]);
		fprintf(stderr, "Apply command to <file>\n"
			"cmdflag\t1 for create cow of <file>\n"
			"\t2 for get indir\n"
			"\t3 for isredir\n"
			"\t4 for delete\n"
			"\t5 for snap_print for an index\n"
			"\t6 for snap_delete for an index\n"
			"\t7 for set block level cow\n"
			"\t8 for restore indirect for an index\n"
				);
		return 1;
	}
	val = strtoul(argv[2], 0, 0);
	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		perror("initial fd error");
		return 1;
	}
	switch(val) {
	case 1: ioc = EXT2_IOC_CREATE_INDIR; 
		err = ioctl(fd, ioc, &ino);
		printf("[%s] create indirect %lu\n",argv[1], ino);
		break;
	case 2: 
		ioc = EXT2_IOC_GET_INDIR; 
		ino = 1;
		if(argv[3])	ino = atoi(argv[3]);
		index = ino;
		err = ioctl(fd, ioc, &ino);
		printf("[%s] get indirect %lu for index %d\n",argv[1], ino,index );
		break;
	case 3: ioc = EXT2_IOC_IS_REDIR; 
		err = ioctl(fd, ioc, &ino);
		printf("[%s] is %s redirector\n", argv[1], ino ? "":"not");
		break;
	case 4: 
		ioc = EXT2_IOC_DESTROY_INDIR; 
		err = ioctl(fd, ioc, &ino);
		break;
	case 5: 
		ioc = EXT2_IOC_SNAP_PRINT; 
		err = ioctl(fd, ioc, &index);
		break;
	case 6: 
		ioc = EXT2_IOC_SNAP_DELETE; 
		err = ioctl(fd, ioc, &index);
		break;
	case 7: ioc = EXT2_IOC_SNAP_SETBLOCKCOW;
		err = ioctl(fd, ioc);
		printf("set block level cow\n");
		break;
	case 8: 
		ioc = EXT2_IOC_RESTORE_INDIR; 
		err = ioctl(fd, ioc, &ino);
		break;

	default: printf("invalid value %ld\n", val); return 2;
	}

	if (err) {
		perror("ioctl failed");
		return 1;
	}

	close(fd);
	return(err);
}

