#!/bin/sh

SNAPFS_O=../snapfs22/snapfs.o
CURRENT=/mnt/current
CLONE=/mnt/clone
SNAPFILE=/tmp/fssnap
SNAPFILE_SIZE=10k
SNAPCTL=../util/snapctl
CURR_DIR=`pwd`
CURRENT_LOOPDEV=/dev/loop0
CACHE_TYPE="ext2"

if [ X$2 != "X" ]; then
    CACHE_TYPE=$2
fi
FS_MODULE=/usr/src/linux/fs/$CACHE_TYPE/$CACHE_TYPE.o

mount_current() {
    mount -t snap_current -o loop,table=0,cache_type=$CACHE_TYPE $SNAPFILE $CURRENT
}
mount_clone() {
    mount -t snap_clone -o dev=$CURRENT_LOOPDEV,name=clone1 none $CLONE
}

snap_start(){

    if [ X"`lsmod |grep snapfs`" != X"" ]; then
	echo "snapfs module is not removed, please run '$0 stop' first"
	exit
    fi

    if [ ! -f $SNAPFS_O ]; then
        echo "snapfs module $SNAPFS_O doesn't exist, exit" 
        exit
    fi

    if [ ! -f $SNAPCTL ]; then
        echo "snapfs utility $SNAPCTL doesn't exist, exit" 
        exit
    fi

    if [ ! -d $CURRENT ]; then
        mkdir -p $CURRENT
    fi
    if [ ! -d $CLONE ]; then
        mkdir -p $CLONE
    fi

    if [ -f $FS_MODULE ]; then
    	insmod $FS_MODULE
    fi

    if [ -f $SNAPFS_O ]; then
    	insmod $SNAPFS_O
    else
	modprobe snapfs
    fi

# create the original files
    echo "Create sample file -- use file $SNAPFILE"
    rm -f $SNAPFILE
    if [ -f $SNAPFILE ]; then 
	echo "$SNAPFILE exist, exit"
	exit 0
    fi

    if [ ! -f $SNAPFILE ]; then 
	dd if=/dev/zero of=$SNAPFILE bs=1024 count=$SNAPFILE_SIZE >/dev/null 2>/dev/null
	/sbin/mke2fs -F $SNAPFILE >/dev/null 2>/dev/null
    fi

    mount -o loop $SNAPFILE $CURRENT

    if [ X$CACHE_TYPE = X"ext3" ]; then
        cd $CURRENT
        dd if=/dev/zero of=.journal bs=1024 count=1k >/dev/null 2>/dev/null
        chattr +i .journal
        cd /
        umount $CURRENT
        mount -t ext3 -o loop,journal=12 $SNAPFILE $CURRENT
    fi

    cd $CURRENT
    echo "yesterday" > hello
    echo "test" > bye
    echo "a" > a
    echo "b" > b
    ln -s hello link
    mkdir dir
    echo "test" > dir/test
    cd $CURR_DIR
    umount $CURRENT
# set up snaptables, install system, 
    echo "Install snaptable -- one snapshot, index 1, name: clone1 "
    rm -f /etc/snapcontrol/table0
    $SNAPCTL -f <<EOF >/dev/null
snaptable
table0
d
1
a
1
now
clone1
q
y
snapset 0 table0
quit
EOF

    echo "Mount current and clone filesystem "
    mount_current
    mount_clone
    echo "Done"
}

case "$1" in

    start)
	snap_start
    ;;

    stop)
	umount $CLONE
    	umount $CURRENT
    	rmmod snapfs
    	rmmod $CACHE_TYPE
    	rmmod loop
    ;;
    *)
    echo "Usage $0 { start, stop} [ext2 | ext3]"
    exit 1;
    
esac

exit 0
