#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

//#include <linux/ext2_fs.h>
#include </usr/src/linux/include/linux/ext2_fs.h>
#include <ext2fs/ext2fs.h>

#include <sys/ioctl.h>
#include <stdlib.h>
#include <fcntl.h>
#include <mntent.h>

static time_t cmptime = 0;
static char base_dir_name[512];
static char mount_root[512];

/* XXX do this better: use getmntent to iterate, then stat the directory and find the device, 
   break from the loop when stat info and passed device are equal */

#define MOUNT_TABLE "/proc/mounts"

static int get_mount_dir(char *device)
{
	
	FILE *fp;
	struct mntent *me;
	
	strcpy(mount_root, "");

	fp = setmntent(MOUNT_TABLE, "r");
	if (!fp)
		return -1;

	while ((me = getmntent(fp))) {
		if (!strcmp(device, me->mnt_fsname)) {
			strcpy(mount_root, me->mnt_dir);
	                break;
		}
	}
        endmntent(fp);

	return 0;
}


static void search(const char *basename, const char *name)
{
	char filename[256];
	struct stat statbuf;
	DIR *curdir;
	struct dirent *item;

	if( *basename == 0 )
		return;
	if( !strcmp(name, "..") || !strcmp(name, ".") )
		return;

	snprintf( filename, 256, "%s%s", basename, name);
			
	if( lstat(filename, &statbuf) ){
		perror("fastfind:");
		return;
	}

	if( S_ISDIR(statbuf.st_mode)){
		if( is_redirector( filename) && strcmp(base_dir_name, filename))
		{
			printf("Info: skip redirector: %s\n", filename);
			return;
		}
		if( statbuf.st_mtime < cmptime &&
		    statbuf.st_ctime < cmptime ){
			printf("Info: skip old dir: %s\n", filename);
			return;
		}
	
		curdir = opendir(filename);
		if( !curdir ){
			perror("fastfind:");
			return;
		}

		printf("%s\n", filename);

		if(filename[strlen(filename)-1]!='/')
			strcat(filename, "/");
		while( item=readdir(curdir) )
			search(filename, item->d_name);
		closedir(curdir);
	}
	else{
		if( statbuf.st_mtime > cmptime ||
		    statbuf.st_ctime > cmptime )
			printf("%s%s\n", basename, name);
	}
}

static int get_pathname(char *device, ino_t parent, ino_t ino)
{
	static ext2_filsys current_fs = NULL;
        int     retval;
        char    *pathname = NULL;
	char 	fullpath[512];

	retval = ext2fs_open(device, 0, 0, 0, unix_io_manager, &current_fs);

	if(retval) {
		printf("Error open device %s\n", device);
		return -1;
	}

	retval = ext2fs_get_pathname(current_fs, parent, ino, &pathname);
	if(retval) {
		printf("Error get pathname for ino %lu(parent %lu)\n", 
		ino, parent);
	}else {
//		printf("For ino %lu, pathname is %s\n", ino, pathname);
		strcpy(fullpath, mount_root);
		strcat(fullpath, pathname);
		strcpy(base_dir_name, fullpath);
		search(fullpath, "");
	}

	free(pathname);

	ext2fs_close(current_fs);
	return 0;
}

int is_redirector(char *fname)
{
	int fd;
	int err;
	ino_t ino = 0;
	int retval = 0;

        fd = open(fname, O_RDONLY);
        if (fd < 0) {
                perror("initial fd error");
                return 0;
        }

	err = ioctl(fd, EXT2_IOC_IS_REDIR, &ino);
	if(ino)	{
		retval = 1;	
		printf("[%s] is %s redirector\n", fname, ino ? "":"not");
	}
	if (err) {
		perror("ioctl failed");
		retval = 0;
	}

	close(fd);
	return(retval);
}

/*
int main(int argc, char* argv[])
{
	if(argc != 2){
		usage();
		return -1;
	}
	search(argv[1], "");
	return 0;
}
*/

main(int argc, char *argv[])
{
	char *device;
	ino_t parent, ino;

	if( argc < 5)	{
		printf("Usage: %s device parent_ino ino cmptime\n", argv[0]);
		return -1;
	}

	device = argv[1];
	parent = atoi(argv[2]);
	ino = atoi(argv[3]);
	cmptime = atoi(argv[4]);
	
	get_mount_dir(device);
	
	return get_pathname(argv[1], parent, ino);
}
